/**
 * JSR 356 defines Java API for WebSocket 1.0. 
 * It defines a standard-based programming model for creating WebSocket client and server endpoint. 
 * Both kind of endpoints can be created programmatically or using annotations. 
 * 
 * JSR-356 is the Java API for WebSocket specification which is included in Java EE 7
 * see http://search.maven.org/remotecontent?filepath=javax/json/javax.json-api/1.0/javax.json-api-1.0.jar
 * and https://jsonp.java.net/
 */

package net.csn;

import java.io.IOException;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.websocket.ClientEndpoint;
import javax.websocket.CloseReason;
import javax.websocket.CloseReason.CloseCodes;
import javax.websocket.ContainerProvider;
import javax.websocket.DeploymentException;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.WebSocketContainer;

import net.csn.dr2d.SeqType;
import net.csn.dr2d.StgyCall;
import net.csn.dr2d.StgyReply;

/**
 * Implémentation de l'interface d'échange 'WebSocket'.
 * Assure le transfert des messages entre le client (la console) et le serveur (le jeu).
 */
@ClientEndpoint
public class WebSocketClient {

    private static final Logger LOGGER = Logger.getLogger(WebSocketClient.class.getName());

    Session session = null;
    
    /**
     * implémentation des réponses du joueur aux sollicitations du jeu.
     */
    WSCsnlPlayer _WSCsnlPlayer = null;

    
    public WebSocketClient(URI endpointURI) throws DeploymentException, IOException {
    	_WSCsnlPlayer = new WSCsnlPlayer();
        WebSocketContainer container = ContainerProvider.getWebSocketContainer();
        this.session = container.connectToServer(this, endpointURI);
    }

    @OnOpen
    public void onOpen(Session session) {
        LOGGER.log(Level.INFO, String.format("[%s] Session ouverte...", session.getId()));
        this.session = session;
    }

    @OnClose
    public void onClose(Session session, CloseReason reason) {
        LOGGER.log(Level.INFO, String.format("[%s] Session fermée {%s}.", session.getId(), reason.toString()));
        this.session = null;
        java.lang.System.exit(0);
    }
    
    @OnError
    public void onError(Throwable t) {
        LOGGER.log(Level.SEVERE, String.format("[%s] Session en erreur :", session.getId()), t);
        java.lang.System.exit(1);
    }

    @OnMessage
    public void onMessage(String message, Session session) {
    	// à la réception d'un message, on le décortique pour en faire une instance de StgyCall
        LOGGER.log(Level.INFO, String.format("[%s] message recu =>\n%s", session.getId(), message));
    	StgyCall stgyCall = StgyCall.parseFrom(message);
    	if(stgyCall!=null){
    		// on construit un réponse appropriée 
    		StgyReply stgyReply = _WSCsnlPlayer.acknowledge(stgyCall);
    		if(stgyReply!=null){
        		// puis on la retourne 
    			final String reply = stgyReply.toJSonString();
    	        LOGGER.log(Level.INFO, String.format("[%s] réponse retournée =>\n%s", session.getId(), reply));
    			sendMessage(session, reply);
    		}
    		// Ceci fermera l'application à la fin de la session de jeu.
    		if(stgyCall.getSequence() == SeqType.GameFinalisation)
				try {
					session.close(new CloseReason(CloseCodes.NORMAL_CLOSURE,"GameFinalisation"));
				} catch (IOException e) {e.printStackTrace();}
    	}
    }

    void sendMessage(Session session, String message) {
        if (session != null) {
            session.getAsyncRemote().sendText(message);
        }
    }
}
